/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! COPYRIGHT    (c)  2005 CAHA, Spain
! IDENT        extract_aper.c
! LANGUAGE     C
! AUTHOR       S.F.Sanchez
! KEYWORDS     
! PURPOSE      C version of extract_aper.c
! COMMENT      No GROUP information included. No header data properly copied
! VERSION      0.1  08.08.05 : Creation, SF.
------------------------------------------------------------------------------*/

#include <IFU_io.h>

#include "stats.h"
#include <stdlib.h>



#define NB_SPEC 256
#define NPIX 1024





main(int argc, char *ARGV[])
{

  
    IMAGE2D image,trace,output;
    
    float *array,*tmp_array;
    char infile[80],trace_file[80],out_file[80];
    int spec_axis,i,k,q,j_min,j_max,j;
    float y_shift,a,in_array,w;
    float peak_y_max_new,range,flux,sum_w;

    char **argval, **arglabel;
    int npix[2];
    double start[2];
    double step[2];
    char tmp_char[80];
    float aperture,fwhm,delta,sub;

    float *a_out,*a_out_new;

    int ny,n_conver;
    int *conver;
    float *we;
    int niter,niter_max;
    float f;
    float c1,c2;
    
    if (argc<8) {
	printf("USE: extract_aper RAW.fits Spectral_axis[0/1] TRACE.fits Aperture OUTPUT.fits FWHM MAX_iter [SHIFT]\n");
	return(0);
    }
    
    init_session(ARGV,argc,&arglabel,&argval);

    strcpy(infile,ARGV[1]);
    spec_axis=atoi(ARGV[2]);
    strcpy(trace_file,ARGV[3]);
    aperture=atof(ARGV[4]);
    strcpy(out_file,ARGV[5]);
    fwhm=atof(ARGV[6]);
//    delta=atof(ARGV[7]);
    niter_max=atoi(ARGV[7]);

    y_shift=0;
    if (argc==9) {
	y_shift=atof(ARGV[8]);
    }

    c1=(exp(-0.5*(pow((aperture)/(fwhm/2.345),2)))/(2.5066*(fwhm/2.345)));

    printf("Reading file infile '%s'",infile);
    open_frame(&image,infile,"I");
    printf("DONE\n");
    printf("Reading file trace_file '%s'",trace_file);
    open_frame(&trace,trace_file,"I");
    printf("DONE\n");
    printf("Creating the output file '%s'",out_file);

    if (spec_axis==0) {
	if (trace.nx!=image.nx) {
	    printf("The X-dimension of the trace file (trace_file), nx_tr\n");
	    printf("does not match that of the original image, nx\n");
	    printf("%d!=%d",trace.nx,image.nx);
	    return(0);
	}    
	ny=image.ny;
    } else {
	if (trace.nx!=image.ny) {
	    printf("The Y-dimension of the trace file (trace_file), nx_tr\n");
	    printf("does not match that of the original image, nx\n");
	    printf("%d!=%d",trace.nx,image.nx);
	    return(0);
	}    
	ny=image.nx;
    }
    a_out=(float *)malloc((trace.nx*trace.ny)*sizeof(float));
    a_out_new=(float *)malloc((trace.nx*trace.ny)*sizeof(float));
    we=(float *)malloc((ny*trace.ny)*sizeof(float));
    conver=(int *)malloc((trace.ny)*sizeof(int));

    npix[0]=image.nx;
    npix[1]=trace.ny;
    start[0]=0.0;
    start[1]=0.0;
    step[0]=1.0;
    step[1]=1.0;

//    printf("DONE\n");

    


    a=0.5*aperture;
    printf("Extracting...\n");


    for (i=0;i<trace.nx;i++) {
	for (k=0;k<trace.ny;k++) {
	    peak_y_max_new=RD_frame(&trace,i,k)+y_shift;	
	    range=(int)(1.5*aperture);
	    j_min=(int)(peak_y_max_new-1.5*aperture);
	    j_max=(int)(peak_y_max_new+1.5*aperture);
	    if (j_min<0) {
		j_min=0;
	    }
	    if (j_max>=image.ny) {
		j_max=image.ny-1;
	    }
	    flux=0;
	    sum_w=0;
	    q=peak_y_max_new;
	    for (j=j_min;j<j_max;j++) {
		if (spec_axis==0) {
		    in_array=RD_frame(&image,i,j);
		} else {
		    in_array=RD_frame(&image,j,i);
		}
		w=0;
		if (((q-a)<=(j-0.5))&&((q+a)>=(j+0.5))) {
		    w=1;
		}
		if (((q-a)<=(j-0.5))&&((q+a)<(j+0.5))) {
		    w=q+a-(j-0.5);
		}
		if (((q-a)>(j-0.5))&&((q+a)>=(j+0.5))) {
		    w=j+0.5-(q-a);
		}
		if (w<0) {
		    w=0;
		}
		sum_w=sum_w+w;
		flux=flux+w*in_array;
	    }
	    a_out[i+trace.nx*k]=flux;
	    a_out_new[i+trace.nx*k]=flux;
	    for (j=0;j<ny;j++) {
		we[j+k*ny]=(exp(-0.5*(pow((j*1.0-peak_y_max_new)/(fwhm/2.345),2)))/(2.5066*(fwhm/2.345)));
	    }
	    conver[k]=0;
	    n_conver=0;
	}
	niter=0;
    }
    printf("Cleaning cross-talk...\n");
    for (i=0;i<trace.nx;i++) {
	n_conver=0;
	niter=0;
	while (niter<niter_max) {
	    for (k=0;k<trace.ny;k++) {	
		if (conver[k]==0) {
		    peak_y_max_new=RD_frame(&trace,i,k)+y_shift;	
		    range=(int)(1.5*aperture);
		    j_min=(int)(peak_y_max_new-range);
		    j_max=(int)(peak_y_max_new+range);
		    if (j_min<0) {
			j_min=0;
		    }
		    if (j_max>=ny) {
			j_max=ny-1;
		    }
		    for (j=j_min;j<j_max;j++) {
			if (spec_axis==0) {
			    in_array=RD_frame(&image,i,j);
		    } else {
			in_array=RD_frame(&image,j,i);
		    }
			f=we[j+k*ny]/(1+niter);
			if (f>0) {
			    if (k>0.0) {
//				sub=f*we[j+(k-1)*ny]*a_out[i+(k-1)*trace.nx];
//				if (sub>delta) {
				    a_out_new[i+trace.nx*k]=a_out_new[i+trace.nx*k]-f*we[j+(k-1)*ny]*a_out[i+(k-1)*trace.nx];	
				    a_out[i+(k-1)*trace.nx]=a_out[i+(k-1)*trace.nx]+f*we[j+(k-1)*ny]*a_out[i+(k-1)*trace.nx];	
//				}
			    }
			    if (k<(trace.ny-1)) {
//				sub=f*we[j+(k+1)*ny]*a_out[i+(k+1)*trace.nx];
//				if (sub>delta) {
				    a_out_new[i+trace.nx*k]=a_out_new[i+trace.nx*k]-f*we[j+(k+1)*ny]*a_out[i+(k+1)*trace.nx];	
				    a_out_new[i+trace.nx*(k+1)]=a_out_new[i+trace.nx*(k+1)]+f*we[j+(k+1)*ny]*a_out[i+(k+1)*trace.nx];	
//				}
			    }		
			}
		    }
		    /*
		    if (abs(a_out_new[i+trace.nx*k]-a_out[i+trace.nx*k])<delta) {
			conver[k]=1;
			n_conver++;
		    }
		    */
		}
	    }
	    for (k=0;k<trace.ny;k++) {	
		a_out[i+trace.nx*k]=a_out_new[i+trace.nx*k];
	    }
	    niter++;
//	    printf("%d %d\n",i,niter);
	}
		    
	if (i==50*nearbyint(i/50)) {
	    printf("%d,%d\n",i,image.nx);
	}
    }





/*
  Saving the results
*/
    create_frame(&output,out_file,npix,start,step,FLOAT,"EXTRACTED",NULL);
    for (i=0;i<trace.nx;i++) {
	for (k=0;k<trace.ny;k++) {    
	    WR_frame(&output,i,k,a_out_new[i+trace.nx*k]);
//	    WR_frame(&output,i,k,a_out[i+trace.nx*k]);
	}
    }
    close_frame(&output);    
    printf("DONE\n");
    exit_session(0);
    return(0);	
}



